#include "plugin.h"
#include "instr_logger.h"
#include <string>
#include <Windows.h>
#include <commdlg.h>
#include <commctrl.h> // Для поддержки Tooltips
#include <algorithm>  // For min and max functions
#include "../pluginsdk/_scriptapi_register.h"
#include "../pluginsdk/bridgemain.h" // For GUI functions
#include "../pluginsdk/_dbgfunctions.h"

// Global variables
static int g_hPlugin = 0;
static int g_hMenu = 0;
static int g_hSettingsDialog = 0;
static HWND g_hWndParent = NULL; // Parent window handle for settings dialog

// External DebugLog function
extern void DebugLog(const char *format, ...);

// Dialog and control IDs
#define IDD_SETTINGS 1000
#define IDC_CALLS_CHECK 1001
#define IDC_JUMPS_CHECK 1002
#define IDC_RETURNS_CHECK 1003
#define IDC_MODULE_CHECK 1004
#define IDC_FOLLOW_CHECK 1005
#define IDC_FILE_CHECK 1006
#define IDC_FILE_PATH 1007
#define IDC_BROWSE_BTN 1008
#define IDC_OK_BTN 1009
#define IDC_CANCEL_BTN 1010

// Forward declarations
void ShowSettingsDialog(HWND hWndParent);
void ShowSettingsGuiCallback();

// GUI callback function for showing settings dialog
void ShowSettingsGuiCallback()
{
    ShowSettingsDialog(g_hWndParent);
}

// Dialog procedure
LRESULT CALLBACK SettingsDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static int yPos = 0;       // Current scroll position
    static int yMinScroll = 0; // Minimum scroll position
    static int yMaxScroll = 0; // Maximum scroll position
    static int yInc = 0;       // Scroll increment

    // Add more detailed tracking of message processing
    if (uMsg == WM_INITDIALOG)
    {
        DebugLog("[%s] *** WM_INITDIALOG received for dialog %p ***\n", PLUGIN_NAME, (void *)hwndDlg);
    }

    switch (uMsg)
    {
    case WM_INITDIALOG:
        // Initialize dialog controls with current settings
        if (g_instrLogger)
        {
            // First reload settings from INI to ensure we have latest values
            DebugLog("[%s] Dialog WM_INITDIALOG: Reloading settings from INI file\n", PLUGIN_NAME);
            g_instrLogger->ReloadSettings();

            DebugLog("[%s] Dialog WM_INITDIALOG: Setting initial control values\n", PLUGIN_NAME);
            const LogOptions &options = g_instrLogger->GetOptions();

            // Dump current options to log
            DebugLog("[%s] Current options to load into dialog:\n", PLUGIN_NAME);
            DebugLog("[%s]   Log CALL instructions: %s\n", PLUGIN_NAME, options.logCalls ? "Yes" : "No");
            DebugLog("[%s]   Log JMP instructions: %s\n", PLUGIN_NAME, options.logJumps ? "Yes" : "No");
            DebugLog("[%s]   Log RET instructions: %s\n", PLUGIN_NAME, options.logReturns ? "Yes" : "No");
            DebugLog("[%s]   Log only current module: %s\n", PLUGIN_NAME, options.logOnlyCurrentModule ? "Yes" : "No");
            DebugLog("[%s]   Follow in CPU: %s\n", PLUGIN_NAME, options.followInCpu ? "Yes" : "No");
            DebugLog("[%s]   Log to file: %s\n", PLUGIN_NAME, options.logToFile ? "Yes" : "No");
            DebugLog("[%s]   Output file: %s\n", PLUGIN_NAME, options.outputFile.c_str());

            // Get control handles for reference
            HWND hCallsCheck = GetDlgItem(hwndDlg, IDC_CALLS_CHECK);
            HWND hJumpsCheck = GetDlgItem(hwndDlg, IDC_JUMPS_CHECK);
            HWND hReturnsCheck = GetDlgItem(hwndDlg, IDC_RETURNS_CHECK);
            HWND hModuleCheck = GetDlgItem(hwndDlg, IDC_MODULE_CHECK);
            HWND hFollowCheck = GetDlgItem(hwndDlg, IDC_FOLLOW_CHECK);
            HWND hFileCheck = GetDlgItem(hwndDlg, IDC_FILE_CHECK);
            HWND hFilePath = GetDlgItem(hwndDlg, IDC_FILE_PATH);
            HWND hBrowseBtn = GetDlgItem(hwndDlg, IDC_BROWSE_BTN);

            // Check if controls are valid
            if (!hCallsCheck || !hJumpsCheck || !hReturnsCheck ||
                !hModuleCheck || !hFollowCheck || !hFileCheck ||
                !hFilePath || !hBrowseBtn)
            {
                DebugLog("[%s] ERROR: One or more dialog controls not found!\n", PLUGIN_NAME);

                // Log the actual control handles for debugging
                DebugLog("[%s] Control handles: Calls=%p, Jumps=%p, Returns=%p, Module=%p, Follow=%p, File=%p, Path=%p, Browse=%p\n",
                         PLUGIN_NAME, hCallsCheck, hJumpsCheck, hReturnsCheck, hModuleCheck, hFollowCheck, hFileCheck, hFilePath, hBrowseBtn);
            }
            else
            {
                // Set checkboxes based on current settings - use direct Windows API calls
                CheckDlgButton(hwndDlg, IDC_CALLS_CHECK, options.logCalls ? BST_CHECKED : BST_UNCHECKED);
                DebugLog("[%s] Set IDC_CALLS_CHECK to %s using CheckDlgButton\n", PLUGIN_NAME, options.logCalls ? "checked" : "unchecked");

                CheckDlgButton(hwndDlg, IDC_JUMPS_CHECK, options.logJumps ? BST_CHECKED : BST_UNCHECKED);
                DebugLog("[%s] Set IDC_JUMPS_CHECK to %s using CheckDlgButton\n", PLUGIN_NAME, options.logJumps ? "checked" : "unchecked");

                CheckDlgButton(hwndDlg, IDC_RETURNS_CHECK, options.logReturns ? BST_CHECKED : BST_UNCHECKED);
                DebugLog("[%s] Set IDC_RETURNS_CHECK to %s using CheckDlgButton\n", PLUGIN_NAME, options.logReturns ? "checked" : "unchecked");

                CheckDlgButton(hwndDlg, IDC_MODULE_CHECK, options.logOnlyCurrentModule ? BST_CHECKED : BST_UNCHECKED);
                DebugLog("[%s] Set IDC_MODULE_CHECK to %s using CheckDlgButton\n", PLUGIN_NAME, options.logOnlyCurrentModule ? "checked" : "unchecked");

                CheckDlgButton(hwndDlg, IDC_FOLLOW_CHECK, options.followInCpu ? BST_CHECKED : BST_UNCHECKED);
                DebugLog("[%s] Set IDC_FOLLOW_CHECK to %s using CheckDlgButton\n", PLUGIN_NAME, options.followInCpu ? "checked" : "unchecked");

                CheckDlgButton(hwndDlg, IDC_FILE_CHECK, options.logToFile ? BST_CHECKED : BST_UNCHECKED);
                DebugLog("[%s] Set IDC_FILE_CHECK to %s using CheckDlgButton\n", PLUGIN_NAME, options.logToFile ? "checked" : "unchecked");

                // Set file path
                SetDlgItemTextA(hwndDlg, IDC_FILE_PATH, options.outputFile.c_str());
                DebugLog("[%s] Set IDC_FILE_PATH to %s\n", PLUGIN_NAME, options.outputFile.c_str());

                // Enable/disable file path controls based on log to file checkbox
                EnableWindow(hFilePath, options.logToFile);
                EnableWindow(hBrowseBtn, options.logToFile);
                DebugLog("[%s] %s file path controls\n", PLUGIN_NAME, options.logToFile ? "Enabled" : "Disabled");

                // Verify values were set correctly
                DebugLog("[%s] Verifying dialog control settings:\n", PLUGIN_NAME);
                UINT callsState = IsDlgButtonChecked(hwndDlg, IDC_CALLS_CHECK);
                UINT jumpsState = IsDlgButtonChecked(hwndDlg, IDC_JUMPS_CHECK);
                UINT returnsState = IsDlgButtonChecked(hwndDlg, IDC_RETURNS_CHECK);
                UINT moduleState = IsDlgButtonChecked(hwndDlg, IDC_MODULE_CHECK);
                UINT followState = IsDlgButtonChecked(hwndDlg, IDC_FOLLOW_CHECK);
                UINT fileState = IsDlgButtonChecked(hwndDlg, IDC_FILE_CHECK);

                DebugLog("[%s] Verified IDC_CALLS_CHECK: %s\n", PLUGIN_NAME,
                         callsState == BST_CHECKED ? "checked" : "unchecked");
                DebugLog("[%s] Verified IDC_JUMPS_CHECK: %s\n", PLUGIN_NAME,
                         jumpsState == BST_CHECKED ? "checked" : "unchecked");
                DebugLog("[%s] Verified IDC_RETURNS_CHECK: %s\n", PLUGIN_NAME,
                         returnsState == BST_CHECKED ? "checked" : "unchecked");
                DebugLog("[%s] Verified IDC_MODULE_CHECK: %s\n", PLUGIN_NAME,
                         moduleState == BST_CHECKED ? "checked" : "unchecked");
                DebugLog("[%s] Verified IDC_FOLLOW_CHECK: %s\n", PLUGIN_NAME,
                         followState == BST_CHECKED ? "checked" : "unchecked");
                DebugLog("[%s] Verified IDC_FILE_CHECK: %s\n", PLUGIN_NAME,
                         fileState == BST_CHECKED ? "checked" : "unchecked");

                char filePath[MAX_PATH] = "";
                GetDlgItemTextA(hwndDlg, IDC_FILE_PATH, filePath, MAX_PATH);
                DebugLog("[%s] Verified IDC_FILE_PATH: %s\n", PLUGIN_NAME, filePath);
            }
        }
        else
        {
            // If g_instrLogger is not available, log an error
            DebugLog("[%s] ERROR: g_instrLogger is NULL in WM_INITDIALOG!\n", PLUGIN_NAME);

            // Try to initialize with default values
            HWND hCallsCheck = GetDlgItem(hwndDlg, IDC_CALLS_CHECK);
            HWND hJumpsCheck = GetDlgItem(hwndDlg, IDC_JUMPS_CHECK);
            HWND hReturnsCheck = GetDlgItem(hwndDlg, IDC_RETURNS_CHECK);
            HWND hModuleCheck = GetDlgItem(hwndDlg, IDC_MODULE_CHECK);
            HWND hFollowCheck = GetDlgItem(hwndDlg, IDC_FOLLOW_CHECK);
            HWND hFileCheck = GetDlgItem(hwndDlg, IDC_FILE_CHECK);
            HWND hFilePath = GetDlgItem(hwndDlg, IDC_FILE_PATH);
            HWND hBrowseBtn = GetDlgItem(hwndDlg, IDC_BROWSE_BTN);

            if (hCallsCheck && hJumpsCheck && hReturnsCheck &&
                hModuleCheck && hFollowCheck && hFileCheck &&
                hFilePath && hBrowseBtn)
            {
                // Set default values using CheckDlgButton
                CheckDlgButton(hwndDlg, IDC_CALLS_CHECK, BST_CHECKED);
                CheckDlgButton(hwndDlg, IDC_JUMPS_CHECK, BST_CHECKED);
                CheckDlgButton(hwndDlg, IDC_RETURNS_CHECK, BST_CHECKED);
                CheckDlgButton(hwndDlg, IDC_MODULE_CHECK, BST_CHECKED);
                CheckDlgButton(hwndDlg, IDC_FOLLOW_CHECK, BST_CHECKED);
                CheckDlgButton(hwndDlg, IDC_FILE_CHECK, BST_UNCHECKED);

                // Disable file path controls by default
                EnableWindow(hFilePath, FALSE);
                EnableWindow(hBrowseBtn, FALSE);

                DebugLog("[%s] Set default values for controls due to missing g_instrLogger\n", PLUGIN_NAME);
            }
            else
            {
                DebugLog("[%s] ERROR: Failed to get dialog controls!\n", PLUGIN_NAME);
            }
        }

        // Initialize scrolling parameters
        yPos = 0;
        yMinScroll = 0;
        yMaxScroll = 300; // This will be updated by WM_USER+100 message
        yInc = 10;        // Scroll increment
        return TRUE;

    // Custom message to update max scroll position
    case WM_USER + 100:
        yMaxScroll = lParam;
        return 0;

    case WM_GETMINMAXINFO:
    {
        // Set minimum window size
        MINMAXINFO *mmi = (MINMAXINFO *)lParam;
        mmi->ptMinTrackSize.x = 600; // Minimum width (increased from 450)
        mmi->ptMinTrackSize.y = 500; // Minimum height (increased from 400)
        return 0;
    }

    case WM_VSCROLL:
    {
        int yNewPos;
        switch (LOWORD(wParam))
        {
        case SB_PAGEUP:
            yNewPos = yPos - 50;
            break;
        case SB_PAGEDOWN:
            yNewPos = yPos + 50;
            break;
        case SB_LINEUP:
            yNewPos = yPos - yInc;
            break;
        case SB_LINEDOWN:
            yNewPos = yPos + yInc;
            break;
        case SB_THUMBTRACK:
            yNewPos = HIWORD(wParam);
            break;
        default:
            yNewPos = yPos;
        }

        // Make sure we don't scroll outside the valid range
        yNewPos = (yNewPos < yMinScroll) ? yMinScroll : ((yNewPos > yMaxScroll) ? yMaxScroll : yNewPos);

        // If position changed, scroll the window and update the position
        if (yNewPos != yPos)
        {
            // Calculate scroll amount
            int scrollAmount = yPos - yNewPos;
            yPos = yNewPos;

            // Scroll all controls except bottom buttons and separator
            EnumChildWindows(hwndDlg, [](HWND hWnd, LPARAM lParam) -> BOOL
                             {
                    // Get control ID
                    int ctrlId = GetWindowLong(hWnd, GWL_ID);

                    // Skip OK and Cancel buttons and separator
                    if (ctrlId == IDC_OK_BTN || ctrlId == IDC_CANCEL_BTN)
                        return TRUE;

                    // Get position of the control
                    RECT rect;
                    GetWindowRect(hWnd, &rect);
                    MapWindowPoints(NULL, GetParent(hWnd), (POINT*)&rect, 2);

                    // Get parent dialog rect to identify bottom area
                    HWND hParent = GetParent(hWnd);
                    RECT parentRect;
                    GetClientRect(hParent, &parentRect);

                    // If control is in the bottom area (near the buttons), don't scroll it
                    if (rect.bottom > parentRect.bottom - 60)
                        return TRUE;

                    // Apply scrolling
                    int* scrollAmount = (int*)lParam;
                    SetWindowPos(hWnd, NULL, rect.left, rect.top + *scrollAmount, 0, 0,
                                 SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
                return TRUE; }, (LPARAM)&scrollAmount);

            // Update scroll position
            SetScrollPos(hwndDlg, SB_VERT, yPos, TRUE);

            // Force a complete redraw to prevent artifacts
            InvalidateRect(hwndDlg, NULL, TRUE);
        }
    }
        return 0;

    case WM_MOUSEWHEEL:
    {
        int wheelDelta = GET_WHEEL_DELTA_WPARAM(wParam);
        int scrollLines = 3; // Number of lines to scroll with each wheel rotation

        // Calculate new position
        int yNewPos = yPos - (wheelDelta / WHEEL_DELTA) * scrollLines * yInc;
        // yNewPos = std::max(yMinScroll, std::min(yNewPos, yMaxScroll));
        yNewPos = (yNewPos < yMinScroll) ? yMinScroll : ((yNewPos > yMaxScroll) ? yMaxScroll : yNewPos);

        if (yNewPos != yPos)
        {
            // Calculate scroll amount
            int scrollAmount = yPos - yNewPos;
            yPos = yNewPos;

            // Scroll all controls except bottom buttons and separator
            EnumChildWindows(hwndDlg, [](HWND hWnd, LPARAM lParam) -> BOOL
                             {
                    // Get control ID
                    int ctrlId = GetWindowLong(hWnd, GWL_ID);

                    // Skip OK and Cancel buttons and separator
                    if (ctrlId == IDC_OK_BTN || ctrlId == IDC_CANCEL_BTN)
                        return TRUE;

                    // Get position of the control
                    RECT rect;
                    GetWindowRect(hWnd, &rect);
                    MapWindowPoints(NULL, GetParent(hWnd), (POINT*)&rect, 2);

                    // Get parent dialog rect to identify bottom area
                    HWND hParent = GetParent(hWnd);
                    RECT parentRect;
                    GetClientRect(hParent, &parentRect);

                    // If control is in the bottom area (near the buttons), don't scroll it
                    if (rect.bottom > parentRect.bottom - 60)
                        return TRUE;

                    // Apply scrolling
                    int* scrollAmount = (int*)lParam;
                    SetWindowPos(hWnd, NULL, rect.left, rect.top + *scrollAmount, 0, 0,
                                 SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
                return TRUE; }, (LPARAM)&scrollAmount);

            // Update scroll position
            SetScrollPos(hwndDlg, SB_VERT, yPos, TRUE);

            // Force a complete redraw to prevent artifacts
            InvalidateRect(hwndDlg, NULL, TRUE);
        }
    }
        return 0;

    case WM_SIZE:
    {
        // Get new client area
        RECT clientRect;
        GetClientRect(hwndDlg, &clientRect);
        int newWidth = clientRect.right;
        int newHeight = clientRect.bottom;

        // Get buttons for repositioning
        HWND hOkBtn = GetDlgItem(hwndDlg, IDC_OK_BTN);
        HWND hCancelBtn = GetDlgItem(hwndDlg, IDC_CANCEL_BTN);
        HWND hFilePath = GetDlgItem(hwndDlg, IDC_FILE_PATH);
        HWND hBrowseBtn = GetDlgItem(hwndDlg, IDC_BROWSE_BTN);

        if (hOkBtn && hCancelBtn)
        {
            // Get button dimensions
            RECT okBtnRect, cancelBtnRect;
            GetWindowRect(hOkBtn, &okBtnRect);
            GetWindowRect(hCancelBtn, &cancelBtnRect);

            int btnWidth = okBtnRect.right - okBtnRect.left;
            int btnHeight = okBtnRect.bottom - okBtnRect.top;

            // Reposition OK and Cancel buttons at the bottom right
            SetWindowPos(hOkBtn, NULL,
                         newWidth - 10 - btnWidth * 2 - 10, newHeight - 10 - btnHeight,
                         0, 0, SWP_NOSIZE | SWP_NOZORDER);

            SetWindowPos(hCancelBtn, NULL,
                         newWidth - 10 - btnWidth, newHeight - 10 - btnHeight,
                         0, 0, SWP_NOSIZE | SWP_NOZORDER);
        }

        // Resize edit control and reposition browse button for file path
        if (hFilePath && hBrowseBtn)
        {
            // Get browse button dimensions
            RECT browseBtnRect;
            GetWindowRect(hBrowseBtn, &browseBtnRect);
            MapWindowPoints(NULL, hwndDlg, (POINT *)&browseBtnRect, 2);

            int btnWidth = browseBtnRect.right - browseBtnRect.left;
            int editHeight = browseBtnRect.bottom - browseBtnRect.top;

            // Get current edit control position
            RECT editRect;
            GetWindowRect(hFilePath, &editRect);
            MapWindowPoints(NULL, hwndDlg, (POINT *)&editRect, 2);

            // Resize edit control to match new width
            SetWindowPos(hFilePath, NULL,
                         editRect.left, editRect.top,
                         newWidth - editRect.left - btnWidth - 25, editHeight,
                         SWP_NOZORDER);

            // Reposition browse button
            SetWindowPos(hBrowseBtn, NULL,
                         newWidth - btnWidth - 15, browseBtnRect.top,
                         0, 0, SWP_NOSIZE | SWP_NOZORDER);
        }

        // Update scrolling parameters based on new window size
        SCROLLINFO si = {0};
        si.cbSize = sizeof(SCROLLINFO);
        si.fMask = SIF_ALL;
        si.nMin = 0;
        si.nMax = yMaxScroll;
        si.nPage = newHeight;
        si.nPos = yPos;
        SetScrollInfo(hwndDlg, SB_VERT, &si, TRUE);

        // Force a complete redraw to prevent artifacts
        InvalidateRect(hwndDlg, NULL, TRUE);
    }
        return 0;

    case WM_ERASEBKGND:
        // Handle background erasing to prevent flickering
        return TRUE;

    case WM_PAINT:
    {
        PAINTSTRUCT ps;
        HDC hdc = BeginPaint(hwndDlg, &ps);

        // Fill background with dialog color
        RECT clientRect;
        GetClientRect(hwndDlg, &clientRect);
        FillRect(hdc, &clientRect, (HBRUSH)(COLOR_BTNFACE + 1));

        EndPaint(hwndDlg, &ps);
        return 0;
    }

    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case IDC_FILE_CHECK:
            // Enable/disable file path control based on checkbox
            {
                BOOL enable = SendMessage(GetDlgItem(hwndDlg, IDC_FILE_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;
                EnableWindow(GetDlgItem(hwndDlg, IDC_FILE_PATH), enable);
                EnableWindow(GetDlgItem(hwndDlg, IDC_BROWSE_BTN), enable);
            }
            return TRUE;

        case IDC_BROWSE_BTN:
            // Show file dialog
            {
                char filename[MAX_PATH] = "";
                GetDlgItemTextA(hwndDlg, IDC_FILE_PATH, filename, MAX_PATH);

                OPENFILENAMEA ofn;
                ZeroMemory(&ofn, sizeof(ofn));
                ofn.lStructSize = sizeof(ofn);
                ofn.hwndOwner = hwndDlg;
                ofn.lpstrFile = filename;
                ofn.nMaxFile = sizeof(filename);
#ifdef _WIN64
                ofn.lpstrFilter = "Trace Files (*.trace64)\0*.trace64\0CSV Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
#else
                ofn.lpstrFilter = "Trace Files (*.trace32)\0*.trace32\0CSV Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
#endif
                ofn.nFilterIndex = 1;
                ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

                if (GetSaveFileNameA(&ofn))
                {
                    SetDlgItemTextA(hwndDlg, IDC_FILE_PATH, filename);
                }
            }
            return TRUE;

        case IDC_OK_BTN:
        case IDOK:
            // Save settings
            if (g_instrLogger)
            {
                DebugLog("[%s] Settings dialog: Collecting settings from dialog\n", PLUGIN_NAME);
                LogOptions options;

                // Get checkboxes
                options.logCalls = SendMessage(GetDlgItem(hwndDlg, IDC_CALLS_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;
                options.logJumps = SendMessage(GetDlgItem(hwndDlg, IDC_JUMPS_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;
                options.logReturns = SendMessage(GetDlgItem(hwndDlg, IDC_RETURNS_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;
                options.logOnlyCurrentModule = SendMessage(GetDlgItem(hwndDlg, IDC_MODULE_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;
                options.followInCpu = SendMessage(GetDlgItem(hwndDlg, IDC_FOLLOW_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;
                options.logToFile = SendMessage(GetDlgItem(hwndDlg, IDC_FILE_CHECK), BM_GETCHECK, 0, 0) == BST_CHECKED;

                // Get file path
                char filename[MAX_PATH] = "";
                GetDlgItemTextA(hwndDlg, IDC_FILE_PATH, filename, MAX_PATH);
                options.outputFile = filename;

                DebugLog("[%s] Settings dialog: Collected settings:\n", PLUGIN_NAME);
                DebugLog("[%s]   Log CALL instructions: %s\n", PLUGIN_NAME, options.logCalls ? "Yes" : "No");
                DebugLog("[%s]   Log JMP instructions: %s\n", PLUGIN_NAME, options.logJumps ? "Yes" : "No");
                DebugLog("[%s]   Log RET instructions: %s\n", PLUGIN_NAME, options.logReturns ? "Yes" : "No");
                DebugLog("[%s]   Log only current module: %s\n", PLUGIN_NAME, options.logOnlyCurrentModule ? "Yes" : "No");
                DebugLog("[%s]   Follow in CPU: %s\n", PLUGIN_NAME, options.followInCpu ? "Yes" : "No");
                DebugLog("[%s]   Log to file: %s\n", PLUGIN_NAME, options.logToFile ? "Yes" : "No");
                if (options.logToFile)
                    DebugLog("[%s]   Output file: %s\n", PLUGIN_NAME, options.outputFile.c_str());

                // Save options in memory
                DebugLog("[%s] Settings dialog: Calling SetOptions\n", PLUGIN_NAME);
                g_instrLogger->SetOptions(options);

                // Save to INI file
                DebugLog("[%s] Settings dialog: Calling SaveSettings\n", PLUGIN_NAME);
                bool saved = g_instrLogger->SaveSettings();
                DebugLog("[%s] Settings dialog: Options %s to INI file\n", PLUGIN_NAME, saved ? "saved" : "failed to save");

                // Verify that settings were actually saved by reloading and comparing
                DebugLog("[%s] Settings dialog: Verifying settings were correctly saved\n", PLUGIN_NAME);
                LogOptions originalOptions = g_instrLogger->GetOptions(); // Save current state

                // Try to reload settings from the INI file
                bool reloaded = g_instrLogger->ReloadSettings();
                if (reloaded)
                {
                    // Get the reloaded options
                    const LogOptions &reloadedOptions = g_instrLogger->GetOptions();

                    // Compare original to reloaded
                    bool allMatch =
                        (originalOptions.logCalls == reloadedOptions.logCalls) &&
                        (originalOptions.logJumps == reloadedOptions.logJumps) &&
                        (originalOptions.logReturns == reloadedOptions.logReturns) &&
                        (originalOptions.logOnlyCurrentModule == reloadedOptions.logOnlyCurrentModule) &&
                        (originalOptions.followInCpu == reloadedOptions.followInCpu) &&
                        (originalOptions.logToFile == reloadedOptions.logToFile) &&
                        (originalOptions.outputFile == reloadedOptions.outputFile);

                    if (!allMatch)
                    {
                        DebugLog("[%s] ERROR: Settings did not save correctly to INI file!\n", PLUGIN_NAME);
                        DebugLog("[%s] Original vs Reloaded:\n", PLUGIN_NAME);
                        DebugLog("[%s]   Log CALL: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.logCalls ? "Yes" : "No",
                                 reloadedOptions.logCalls ? "Yes" : "No");
                        DebugLog("[%s]   Log JMP: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.logJumps ? "Yes" : "No",
                                 reloadedOptions.logJumps ? "Yes" : "No");
                        DebugLog("[%s]   Log RET: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.logReturns ? "Yes" : "No",
                                 reloadedOptions.logReturns ? "Yes" : "No");
                        DebugLog("[%s]   Log only module: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.logOnlyCurrentModule ? "Yes" : "No",
                                 reloadedOptions.logOnlyCurrentModule ? "Yes" : "No");
                        DebugLog("[%s]   Follow CPU: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.followInCpu ? "Yes" : "No",
                                 reloadedOptions.followInCpu ? "Yes" : "No");
                        DebugLog("[%s]   Log to file: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.logToFile ? "Yes" : "No",
                                 reloadedOptions.logToFile ? "Yes" : "No");
                        DebugLog("[%s]   Output file: %s vs %s\n", PLUGIN_NAME,
                                 originalOptions.outputFile.c_str(),
                                 reloadedOptions.outputFile.c_str());

                        saved = false; // Settings didn't save correctly

                        // Restore the original settings
                        g_instrLogger->SetOptions(originalOptions);
                    }
                    else
                    {
                        DebugLog("[%s] Settings verification successful - values match\n", PLUGIN_NAME);
                    }
                }
                else
                {
                    DebugLog("[%s] ERROR: Could not reload settings for verification!\n", PLUGIN_NAME);
                    saved = false; // Consider it failed if we can't verify
                }

                // Inform the user
                if (saved)
                {
                    _plugin_logprintf("[%s] Settings updated and saved to INI file\n", PLUGIN_NAME);
                }
                else
                {
                    _plugin_logprintf("[%s] WARNING: Failed to save settings to INI file\n", PLUGIN_NAME);
                    MessageBoxA(hwndDlg, "Settings were updated in memory but could not be saved to INI file. Changes will be lost when x64dbg is closed.",
                                "Settings Save Error", MB_OK | MB_ICONWARNING);
                }

                // Display current settings in log for reference
                const LogOptions &updatedOptions = g_instrLogger->GetOptions();
                _plugin_logprintf("[%s] Current settings:\n", PLUGIN_NAME);
                _plugin_logprintf("  Log CALL instructions: %s\n", updatedOptions.logCalls ? "Yes" : "No");
                _plugin_logprintf("  Log JMP instructions: %s\n", updatedOptions.logJumps ? "Yes" : "No");
                _plugin_logprintf("  Log RET instructions: %s\n", updatedOptions.logReturns ? "Yes" : "No");
                _plugin_logprintf("  Log only current module: %s\n", updatedOptions.logOnlyCurrentModule ? "Yes" : "No");
                _plugin_logprintf("  Follow in CPU: %s\n", updatedOptions.followInCpu ? "Yes" : "No");
                _plugin_logprintf("  Log to file: %s\n", updatedOptions.logToFile ? "Yes" : "No");
                if (updatedOptions.logToFile)
                    _plugin_logprintf("  Output file: %s\n", updatedOptions.outputFile.c_str());
            }
            // Закрываем окно диалога без PostQuitMessage
            DebugLog("[%s] Settings dialog: OK button clicked, destroying dialog\n", PLUGIN_NAME);
            DestroyWindow(hwndDlg);
            return TRUE;

        case IDC_CANCEL_BTN:
        case IDCANCEL:
            // Закрываем окно диалога без PostQuitMessage
            DebugLog("[%s] Settings dialog: Cancel button clicked, destroying dialog\n", PLUGIN_NAME);
            DestroyWindow(hwndDlg);
            return TRUE;
        }
        break;

    case WM_CLOSE:
        // Закрываем окно диалога без PostQuitMessage
        DebugLog("[%s] Settings dialog: Close button clicked, destroying dialog\n", PLUGIN_NAME);
        DestroyWindow(hwndDlg);
        return TRUE;

    case WM_DESTROY:
        // Диалог уничтожен, просто логируем это событие
        DebugLog("[%s] Settings dialog: WM_DESTROY received\n", PLUGIN_NAME);
        return TRUE;
    }

    return DefWindowProcA(hwndDlg, uMsg, wParam, lParam);
}

// Check if tracing functionality is available and safe to use
static bool IsTracingAvailable()
{
    // Make sure we have a valid debuggee
    if (!DbgIsDebugging())
    {
        DebugLog("[%s] IsTracingAvailable: Not debugging\n", PLUGIN_NAME);
        return false;
    }

    // Make sure the debugger is not running (must be paused to trace)
    if (DbgIsRunning())
    {
        DebugLog("[%s] IsTracingAvailable: Debugger is running, not paused\n", PLUGIN_NAME);
        return false;
    }

    // Check if we can get the CIP register - if this fails, debugger isn't ready
    duint cip = 0;
    try
    {
        cip = Script::Register::GetCIP();
        if (cip == 0)
        {
            DebugLog("[%s] IsTracingAvailable: CIP is zero\n", PLUGIN_NAME);
            return false;
        }

        // Проверка, что текущая позиция содержит корректную инструкцию
        char disasm[256] = "";
        if (!GuiGetDisassembly(cip, disasm) || disasm[0] == '\0')
        {
            DebugLog("[%s] IsTracingAvailable: Cannot disassemble at current position (CIP=%p)\n", PLUGIN_NAME, (void *)cip);
            return false;
        }
    }
    catch (...)
    {
        DebugLog("[%s] IsTracingAvailable: Exception getting CIP\n", PLUGIN_NAME);
        return false;
    }

    // Everything seems OK
    DebugLog("[%s] IsTracingAvailable: Tracing is available\n", PLUGIN_NAME);
    return true;
}

// Plugin initialization
bool pluginit(PLUG_INITSTRUCT *initStruct)
{
    initStruct->pluginVersion = PLUGIN_VERSION;
    initStruct->sdkVersion = PLUG_SDKVERSION;
    strncpy_s(initStruct->pluginName, PLUGIN_NAME, _TRUNCATE);

    // Save plugin handle
    g_hPlugin = initStruct->pluginHandle;

    DebugLog("[%s] Initializing plugin (handle: %d)\n", PLUGIN_NAME, g_hPlugin);

    // Initialize Common Controls for tooltips
    INITCOMMONCONTROLSEX icex;
    icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    icex.dwICC = ICC_TAB_CLASSES | ICC_TREEVIEW_CLASSES | ICC_BAR_CLASSES;
    InitCommonControlsEx(&icex);
    DebugLog("[%s] Common Controls initialized\n", PLUGIN_NAME);

    // Register callbacks
    _plugin_registercallback(g_hPlugin, CB_BREAKPOINT, (CBPLUGIN)CBBREAKPOINT);
    _plugin_registercallback(g_hPlugin, CB_STEPPED, (CBPLUGIN)CBSTEPPED);
    _plugin_registercallback(g_hPlugin, CB_TRACEEXECUTE, (CBPLUGIN)CBTRACEEXECUTE);
    _plugin_registercallback(g_hPlugin, CB_MENUENTRY, (CBPLUGIN)CBMENUENTRY);
    _plugin_registercallback(g_hPlugin, CB_WINEVENT, (CBPLUGIN)CBWINEVENT);
    DebugLog("[%s] Callbacks registered\n", PLUGIN_NAME);

    // Create instruction logger
    g_instrLogger = new InstrLogger();
    if (!g_instrLogger->Initialize())
    {
        DebugLog("[%s] Failed to initialize instruction logger\n", PLUGIN_NAME);
        return false;
    }

    DebugLog("[%s] Plugin initialized successfully\n", PLUGIN_NAME);
    return true;
}

// Plugin cleanup
void plugstop()
{
    DebugLog("[%s] Plugin stopping\n", PLUGIN_NAME);

    // Clean up resources
    if (g_instrLogger)
    {
        delete g_instrLogger;
        g_instrLogger = nullptr;
        DebugLog("[%s] Instruction logger deleted\n", PLUGIN_NAME);
    }

    DebugLog("[%s] Plugin stopped\n", PLUGIN_NAME);
}

// Plugin setup
void plugsetup(PLUG_SETUPSTRUCT *setupStruct)
{
    DebugLog("[%s] Plugin setup starting\n", PLUGIN_NAME);

    // Add menu items
    g_hMenu = setupStruct->hMenu;

    // Create plugin menu
    int hMenuInstrLogger = _plugin_menuadd(g_hMenu, "&Instruction Logger");
    DebugLog("[%s] Added main menu (handle: %d)\n", PLUGIN_NAME, hMenuInstrLogger);
    // Add menu entries - Logging operations
    _plugin_menuaddentry(hMenuInstrLogger, MENU_START_LOGGING, "▶ &Start Logging");
    _plugin_menuaddentry(hMenuInstrLogger, MENU_STOP_LOGGING, "■ S&top Logging");
    _plugin_menuaddseparator(hMenuInstrLogger);

    _plugin_menuaddentry(hMenuInstrLogger, MENU_CLEAR_LOG, "🗑️ &Clear Log");
    _plugin_menuaddentry(hMenuInstrLogger, MENU_SAVE_LOG, "💾 &Save Log...");

    // Add separator between operations and configuration
    _plugin_menuaddseparator(hMenuInstrLogger);

    // Add configuration entries
    _plugin_menuaddentry(hMenuInstrLogger, MENU_SETTINGS, "⚙️ Se&ttings...");

    // Add separator before help
    _plugin_menuaddseparator(hMenuInstrLogger);

    // Add help entry
    _plugin_menuaddentry(hMenuInstrLogger, MENU_HELP, "❓ &Help");

    DebugLog("[%s] Added menu entries with user-friendly symbols\n", PLUGIN_NAME);

    // Disable stop menu initially
    _plugin_menuentrysetvisible(g_hMenu, MENU_STOP_LOGGING, false);

    // Display settings location
    if (g_instrLogger)
    {
        // Get plugin directory for INI file
        char pluginPath[MAX_PATH] = "";
        std::string iniPath = "Unknown Location";

        if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
            GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
        {
            char *lastSlash = strrchr(pluginPath, '\\');
            if (lastSlash)
            {
                *(lastSlash + 1) = 0; // Truncate to directory
                iniPath = std::string(pluginPath) + "InstrLoggerSettings.ini";
            }
        }

        _plugin_logprintf("[%s] Settings stored in: %s\n", PLUGIN_NAME, iniPath.c_str());

        // If settings were loaded, display them in the log
        const LogOptions &options = g_instrLogger->GetOptions();
        _plugin_logprintf("[%s] Current settings (loaded from INI file):\n", PLUGIN_NAME);
        _plugin_logprintf("  Log CALL instructions: %s\n", options.logCalls ? "Yes" : "No");
        _plugin_logprintf("  Log JMP instructions: %s\n", options.logJumps ? "Yes" : "No");
        _plugin_logprintf("  Log RET instructions: %s\n", options.logReturns ? "Yes" : "No");
        _plugin_logprintf("  Log only current module: %s\n", options.logOnlyCurrentModule ? "Yes" : "No");
        _plugin_logprintf("  Follow in CPU: %s\n", options.followInCpu ? "Yes" : "No");
        _plugin_logprintf("  Log to file: %s\n", options.logToFile ? "Yes" : "No");
        if (options.logToFile)
            _plugin_logprintf("  Output file: %s\n", options.outputFile.c_str());
    }

    DebugLog("[%s] Plugin loaded successfully!\n", PLUGIN_NAME);
    _plugin_logprintf("[%s] Plugin loaded successfully!\n", PLUGIN_NAME);
    _plugin_logprintf("[%s] Use the 'Plugins -> Instruction Logger' menu to log instructions.\n", PLUGIN_NAME);
}

// Breakpoint callback
void CBBREAKPOINT(CBTYPE cbType, PLUG_CB_BREAKPOINT *info)
{
    try
    {
        if (g_instrLogger)
            g_instrLogger->OnBreakpoint(info);
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBBREAKPOINT: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBBREAKPOINT\n", PLUGIN_NAME);
    }
}

// Step callback
void CBSTEPPED(CBTYPE cbType, PLUG_CB_STEPPED *info)
{
    try
    {
        if (g_instrLogger)
            g_instrLogger->OnStepped(info);
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBSTEPPED: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBSTEPPED\n", PLUGIN_NAME);
    }
}

// Trace execute callback
void CBTRACEEXECUTE(CBTYPE cbType, PLUG_CB_TRACEEXECUTE *info)
{
    try
    {
        if (g_instrLogger)
            g_instrLogger->OnTraceExecute(info);
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBTRACEEXECUTE: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBTRACEEXECUTE\n", PLUGIN_NAME);
    }
}

// Menu callback
void CBMENUENTRY(CBTYPE cbType, PLUG_CB_MENUENTRY *info)
{
    if (!g_instrLogger)
        return;

    try
    {
        switch (info->hEntry)
        {
        case MENU_START_LOGGING:
            DebugLog("[%s] Menu: Start Logging selected\n", PLUGIN_NAME);

            // First check if debugging is active
            if (!DbgIsDebugging())
            {
                DebugLog("[%s] Cannot start logging: No debugging session active\n", PLUGIN_NAME);
                _plugin_logprintf("[%s] Cannot start logging: No debugging session active. Please start debugging first.\n", PLUGIN_NAME);
                MessageBoxA(GuiGetWindowHandle(), "Please start a debugging session first.", PLUGIN_NAME, MB_OK | MB_ICONINFORMATION);
                break;
            }

            if (g_instrLogger->StartLogging())
            {
                _plugin_menuentrysetvisible(g_hMenu, MENU_START_LOGGING, false);
                _plugin_menuentrysetvisible(g_hMenu, MENU_STOP_LOGGING, true);
                DebugLog("[%s] Menu visibility updated for Start/Stop\n", PLUGIN_NAME);

                // Show trace dialog to set up trace
                GuiExecuteOnGuiThread([]()
                                      {
                    try {
                        DebugLog("[%s] Setting up trace commands\n", PLUGIN_NAME);

                        // Check if tracing is available before proceeding
                        if (!IsTracingAvailable()) {
                            DebugLog("[%s] Tracing is not available at this time\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Tracing is not available. Make sure you are debugging an application.\n", PLUGIN_NAME);
                            return;
                        }

                        // Дополнительная защитная проверка перед вызовом функций трассировки
                        if (!DbgIsDebugging() || DbgIsRunning()) {
                            DebugLog("[%s] Cannot trace: debugger is not in correct state\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Cannot start trace: debugger must be debugging and paused.\n", PLUGIN_NAME);
                            return;
                        }

                        // Используем более безопасный подход без вызова потенциально опасных функций
                        try {
                            // Настраиваем файл лога через GUI функцию
                            char pluginPath[MAX_PATH] = "";
                            char fullPath[MAX_PATH] = ""; // Define fullPath variable here

                            if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                                GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH)) {

                                char* lastSlash = strrchr(pluginPath, '\\');
                                if (lastSlash) {
                                    *(lastSlash + 1) = 0; // Truncate to directory
#ifdef _WIN64
                                    sprintf_s(fullPath, "%sinstrlog_trace.trace64", pluginPath);
#else
                                    sprintf_s(fullPath, "%sinstrlog_trace.trace32", pluginPath);
#endif
                                    DebugLog("[%s] Using trace log path: %s\n", PLUGIN_NAME, fullPath);

                                    // Используем GUI функцию
                                    GuiOpenTraceFile(fullPath);
                                    DebugLog("[%s] Set trace log file via GUI function\n", PLUGIN_NAME);
                                }
                            } else {
                                // Запасной вариант
#ifdef _WIN64
                                strcpy_s(fullPath, ".\\instrlog_trace.trace64"); // Set default path
                                GuiOpenTraceFile(fullPath);
#else
                                strcpy_s(fullPath, ".\\instrlog_trace.trace32"); // Set default path
                                GuiOpenTraceFile(fullPath);
#endif
                                DebugLog("[%s] Set default trace log file\n", PLUGIN_NAME);
                            }

                            // Открываем и обновляем окно трассировки
                            GuiShowTrace();
                            Sleep(200);
                            GuiUpdateTraceBrowser();

                            // First, set up the trace log file
                            char traceLogFileCmd[MAX_PATH + 32];
                            sprintf_s(traceLogFileCmd, "TraceSetLogFile,\"%s\"", fullPath);
                            if (!DbgCmdExec(traceLogFileCmd)) {
                                DebugLog("[%s] Failed to set trace log file\n", PLUGIN_NAME);
                                return;
                            }

                            // Configure trace logging based on plugin options
                            const LogOptions &options = g_instrLogger->GetOptions();
                            std::string condition = ""; // Define condition at this scope level

                            if (options.logCalls || options.logJumps || options.logReturns) {
                                // Build the condition expression
                                bool first = true;

                                if (options.logCalls) {
                                    condition += "disasm.iscall(cip)";
                                    first = false;
                                }

                                if (options.logJumps) {
                                    if (!first) condition += " || ";
                                    condition += "disasm.isbranch(cip)";
                                    first = false;
                                }

                                if (options.logReturns) {
                                    if (!first) condition += " || ";
                                    condition += "disasm.isret(cip)";
                                    first = false;
                                }

                                // Add module restriction if needed
                                if (options.logOnlyCurrentModule) {
                                    duint modBase = 0, modSize = 0;
                                    char modName[MAX_PATH] = "";

                                    if (DbgGetModuleAt(Script::Register::GetCIP(), modName)) {
                                        // Get module base address
                                        modBase = DbgFunctions()->ModBaseFromName(modName);

                                        // Get module size if we have a valid base address
                                        if (modBase != 0) {
                                            modSize = DbgFunctions()->ModSizeFromAddr(modBase);

                                            if (modSize != 0) {
                                                char rangeCondition[128];
#ifdef _WIN64
                                                sprintf_s(rangeCondition, " && (cip >= 0x%llX && cip < 0x%llX)", modBase, modBase + modSize);
#else
                                                sprintf_s(rangeCondition, " && (cip >= 0x%X && cip < 0x%X)", modBase, modBase + modSize);
#endif
                                                condition += rangeCondition;
                                            }
                                        }
                                    }
                                }

                                // Start trace recording
                                if (!DbgCmdExec("StartTraceRecording")) {
                                    DebugLog("[%s] Failed to start trace recording\n", PLUGIN_NAME);
                                    return;
                                }

                                // Set the trace log condition (what to log)
                                char logCmd[512];
                                sprintf_s(logCmd, "TraceSetLog,\"${rip}: ${dasm}\",\"%s\"", condition.c_str());
                                if (!DbgCmdExec(logCmd)) {
                                    DebugLog("[%s] Failed to set trace log condition\n", PLUGIN_NAME);
                                    return;
                                }

                                // Now start the conditional tracing operation
                                if (DbgCmdExec("TraceIntoConditional 0,50000")) {  // Fixed command format by removing the space after comma
                                    DebugLog("[%s] Trace recording started successfully\n", PLUGIN_NAME);
                                    _plugin_logprintf("[%s] Trace recording started\n", PLUGIN_NAME);
                                    _plugin_logprintf("[%s] Tracing %s%s%s%s%s instructions%s\n", PLUGIN_NAME,
                                        options.logCalls ? "CALL" : "",
                                        (options.logCalls && (options.logJumps || options.logReturns)) ? ", " : "",
                                        options.logJumps ? "JMP" : "",
                                        (options.logJumps && options.logReturns) ? ", " : "",
                                        options.logReturns ? "RET" : "",
                                        options.logOnlyCurrentModule ? " in current module only" : "");
                                } else {
                                    DebugLog("[%s] Failed to start conditional tracing\n", PLUGIN_NAME);
                                    _plugin_logprintf("[%s] Failed to start trace recording. Command format error or debugger not in valid state.\n", PLUGIN_NAME);

                                    // Attempt to provide more detailed error information
                                    DWORD lastError = GetLastError();
                                    if (lastError != 0) {
                                        DebugLog("[%s] System error code: %d\n", PLUGIN_NAME, lastError);
                                        _plugin_logprintf("[%s] System error code: %d\n", PLUGIN_NAME, lastError);
                                    }

                                    // Notify user that tracing failed but logging is still active
                                    _plugin_logprintf("[%s] Basic instruction logging is still active without tracing\n", PLUGIN_NAME);
                                }
                            }

                            DebugLog("[%s] Trace setup completed\n", PLUGIN_NAME);
                        }
                        catch (const std::exception& e) {
                            DebugLog("[%s] Exception in trace setup: %s\n", PLUGIN_NAME, e.what());
                            _plugin_logprintf("[%s] Error setting up trace: %s\n", PLUGIN_NAME, e.what());
                        }
                        catch (...) {
                            DebugLog("[%s] Unknown exception in trace setup\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Unknown error setting up trace\n", PLUGIN_NAME);
                        }
                    }
                    catch (const std::exception& e) {
                        DebugLog("[%s] Exception in trace setup: %s\n", PLUGIN_NAME, e.what());
                    }
                    catch (...) {
                        DebugLog("[%s] Unknown exception in trace setup\n", PLUGIN_NAME);
                    } });
            }
            break;

        case MENU_STOP_LOGGING:
            DebugLog("[%s] Menu: Stop Logging selected\n", PLUGIN_NAME);
            if (g_instrLogger->StopLogging())
            {
                _plugin_menuentrysetvisible(g_hMenu, MENU_START_LOGGING, true);
                _plugin_menuentrysetvisible(g_hMenu, MENU_STOP_LOGGING, false);
                DebugLog("[%s] Menu visibility updated for Start/Stop\n", PLUGIN_NAME);

                // Stop tracing
                GuiExecuteOnGuiThread([]()
                                      {
                    try {
                        DebugLog("[%s] Stopping trace recording\n", PLUGIN_NAME);

                        // Автоматически останавливаем запись трассировки
                        if (DbgCmdExec("StopTraceRecording")) {  // Changed from TraceIntoStop to StopTraceRecording
                            DebugLog("[%s] Trace recording stopped automatically\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Trace recording stopped\n", PLUGIN_NAME);

                            // Автоматически сохраняем трассировку
                            try {
                                char pluginPath[MAX_PATH] = "";
                                if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                                    GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH)) {

                                    char* lastSlash = strrchr(pluginPath, '\\');
                                    if (lastSlash) {
                                        *(lastSlash + 1) = 0; // Truncate to directory
                                        char savePath[MAX_PATH] = "";

                                        // Создаем имя файла с текущей датой и временем
                                        SYSTEMTIME st;
                                        GetLocalTime(&st);

#ifdef _WIN64
                                        sprintf_s(savePath, "%sinstrlog_trace_%04d%02d%02d_%02d%02d%02d.trace64",
                                            pluginPath, st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
#else
                                        sprintf_s(savePath, "%sinstrlog_trace_%04d%02d%02d_%02d%02d%02d.trace32",
                                            pluginPath, st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
#endif

                                        // Сохраняем трассировку
                                        char cmdSave[MAX_PATH + 50] = "";
                                        sprintf_s(cmdSave, "TraceSave,\"%s\"", savePath);

                                        if (DbgCmdExec(cmdSave)) {
                                            DebugLog("[%s] Trace saved to %s\n", PLUGIN_NAME, savePath);
                                            _plugin_logprintf("[%s] Trace saved to %s\n", PLUGIN_NAME, savePath);
                                        } else {
                                            DebugLog("[%s] Failed to save trace to %s\n", PLUGIN_NAME, savePath);
                                            _plugin_logprintf("[%s] Failed to save trace. Please save manually from the trace window\n", PLUGIN_NAME);
                                        }
                                    }
                                }
                            } catch (const std::exception& e) {
                                DebugLog("[%s] Exception when saving trace: %s\n", PLUGIN_NAME, e.what());
                            } catch (...) {
                                DebugLog("[%s] Unknown exception when saving trace\n", PLUGIN_NAME);
                            }
                        }
                        else {
                            DebugLog("[%s] Failed to stop trace recording automatically\n", PLUGIN_NAME);
                            _plugin_logprintf("[%s] Failed to stop trace recording\n", PLUGIN_NAME);
                        }
                    }
                    catch (const std::exception& e) {
                        DebugLog("[%s] Exception stopping trace recording: %s\n", PLUGIN_NAME, e.what());
                    }
                    catch (...) {
                        DebugLog("[%s] Unknown exception stopping trace recording\n", PLUGIN_NAME);
                    } });
            }
            break;

        case MENU_CLEAR_LOG:
            DebugLog("[%s] Menu: Clear Log selected\n", PLUGIN_NAME);
            g_instrLogger->ClearLog();
            break;

        case MENU_SAVE_LOG:
            DebugLog("[%s] Menu: Save Log selected\n", PLUGIN_NAME);
            {
                // Show file dialog
                char filename[MAX_PATH] = "";
                strncpy_s(filename, g_instrLogger->GetOptions().outputFile.c_str(), _TRUNCATE);

                OPENFILENAMEA ofn;
                ZeroMemory(&ofn, sizeof(ofn));
                ofn.lStructSize = sizeof(ofn);
                ofn.hwndOwner = GuiGetWindowHandle();
                ofn.lpstrFile = filename;
                ofn.nMaxFile = sizeof(filename);
#ifdef _WIN64
                ofn.lpstrFilter = "Trace Files (*.trace64)\0*.trace64\0CSV Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
#else
                ofn.lpstrFilter = "Trace Files (*.trace32)\0*.trace32\0CS`Files (*.csv)\0*.csv\0Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
#endif
                ofn.nFilterIndex = 1;
                ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;

                DebugLog("[%s] Showing save file dialog\n", PLUGIN_NAME);
                if (GetSaveFileNameA(&ofn))
                {
                    DebugLog("[%s] User selected save file: %s\n", PLUGIN_NAME, filename);
                    g_instrLogger->SaveLog(filename);
                }
                else
                {
                    DebugLog("[%s] Save file dialog canceled or failed (error: %d)\n", PLUGIN_NAME, CommDlgExtendedError());
                }
            }
            break;

        case MENU_SETTINGS:
            DebugLog("[%s] Menu: Settings selected\n", PLUGIN_NAME);
            if (GuiGetWindowHandle())
            {
                // First verify that the INI directory exists and is writable
                char pluginPath[MAX_PATH] = "";
                std::string iniPath = "";
                bool iniDirOK = false;

                if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                    GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
                {
                    char *lastSlash = strrchr(pluginPath, '\\');
                    if (lastSlash)
                    {
                        *(lastSlash + 1) = 0; // Truncate to directory
                        iniPath = std::string(pluginPath);

                        // Verify directory exists and is writable
                        DWORD attrs = GetFileAttributesA(iniPath.c_str());
                        if (attrs != INVALID_FILE_ATTRIBUTES && (attrs & FILE_ATTRIBUTE_DIRECTORY))
                        {
                            // Test if directory is writable
                            std::string testFile = iniPath + "test_write.tmp";
                            HANDLE hTest = CreateFileA(testFile.c_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
                                                       FILE_ATTRIBUTE_NORMAL, NULL);
                            if (hTest != INVALID_HANDLE_VALUE)
                            {
                                CloseHandle(hTest);
                                DeleteFileA(testFile.c_str());
                                iniDirOK = true;
                                DebugLog("[%s] INI directory verified to exist and be writable: %s\n",
                                         PLUGIN_NAME, iniPath.c_str());
                            }
                            else
                            {
                                DWORD error = GetLastError();
                                DebugLog("[%s] INI directory exists but is not writable: %s (error: %lu)\n",
                                         PLUGIN_NAME, iniPath.c_str(), error);
                            }
                        }
                        else
                        {
                            DWORD error = GetLastError();
                            DebugLog("[%s] INI directory does not exist: %s (error: %lu)\n",
                                     PLUGIN_NAME, iniPath.c_str(), error);

                            // Try to create the directory
                            if (CreateDirectoryA(iniPath.c_str(), NULL) || GetLastError() == ERROR_ALREADY_EXISTS)
                            {
                                iniDirOK = true;
                                DebugLog("[%s] Successfully created INI directory: %s\n", PLUGIN_NAME, iniPath.c_str());
                            }
                            else
                            {
                                error = GetLastError();
                                DebugLog("[%s] Failed to create INI directory: %s (error: %lu)\n",
                                         PLUGIN_NAME, iniPath.c_str(), error);
                            }
                        }
                    }
                }

                if (!iniDirOK)
                {
                    DebugLog("[%s] Problem with INI directory, displaying warning to user\n", PLUGIN_NAME);
                    MessageBoxA(GuiGetWindowHandle(),
                                "Warning: Could not access or create the directory for saving settings.\n"
                                "Settings changes may not persist between sessions.",
                                "InstrLogger Settings Warning", MB_OK | MB_ICONWARNING);
                }

                // Reload settings from INI file before showing the dialog
                if (g_instrLogger)
                {
                    DebugLog("[%s] Reloading settings from INI file before showing settings dialog\n", PLUGIN_NAME);
                    g_instrLogger->ReloadSettings();

                    // Show current settings in log
                    const LogOptions &options = g_instrLogger->GetOptions();
                    DebugLog("[%s] Displaying current settings in log\n", PLUGIN_NAME);
                    _plugin_logprintf("[%s] Current settings:\n", PLUGIN_NAME);
                    _plugin_logprintf("  Log CALL instructions: %s\n", options.logCalls ? "Yes" : "No");
                    _plugin_logprintf("  Log JMP instructions: %s\n", options.logJumps ? "Yes" : "No");
                    _plugin_logprintf("  Log RET instructions: %s\n", options.logReturns ? "Yes" : "No");
                    _plugin_logprintf("  Log only current module: %s\n", options.logOnlyCurrentModule ? "Yes" : "No");
                    _plugin_logprintf("  Follow in CPU: %s\n", options.followInCpu ? "Yes" : "No");
                    _plugin_logprintf("  Log to file: %s\n", options.logToFile ? "Yes" : "No");
                    if (options.logToFile)
                        _plugin_logprintf("  Output file: %s\n", options.outputFile.c_str());

                    // Also log INI file path information
                    _plugin_logprintf("[%s] Settings file: %s\n", PLUGIN_NAME,
                                      !iniPath.empty() ? (iniPath + CUSTOM_INI_FILENAME).c_str() : "Unknown location");
                }

                // Show the settings dialog
                g_hWndParent = GuiGetWindowHandle();
                GuiExecuteOnGuiThread(ShowSettingsGuiCallback);

                DebugLog("[%s] Settings dialog requested\n", PLUGIN_NAME);
            }
            else
            {
                // Fallback if GUI is not available
                DebugLog("[%s] Unable to get GUI window handle, showing settings in log only\n", PLUGIN_NAME);
                if (g_instrLogger)
                {
                    // Reload settings first in case they've changed
                    g_instrLogger->ReloadSettings();

                    const LogOptions &options = g_instrLogger->GetOptions();
                    _plugin_logprintf("[%s] Current settings (GUI not available):\n", PLUGIN_NAME);
                    _plugin_logprintf("  Log CALL instructions: %s\n", options.logCalls ? "Yes" : "No");
                    _plugin_logprintf("  Log JMP instructions: %s\n", options.logJumps ? "Yes" : "No");
                    _plugin_logprintf("  Log RET instructions: %s\n", options.logReturns ? "Yes" : "No");
                    _plugin_logprintf("  Log only current module: %s\n", options.logOnlyCurrentModule ? "Yes" : "No");
                    _plugin_logprintf("  Follow in CPU: %s\n", options.followInCpu ? "Yes" : "No");
                    _plugin_logprintf("  Log to file: %s\n", options.logToFile ? "Yes" : "No");
                    if (options.logToFile)
                        _plugin_logprintf("  Output file: %s\n", options.outputFile.c_str());

                    // Get plugin path to show INI location
                    char pluginPath[MAX_PATH] = "";
                    std::string iniPath = "Unknown Location";

                    if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                        GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
                    {
                        char *lastSlash = strrchr(pluginPath, '\\');
                        if (lastSlash)
                        {
                            *(lastSlash + 1) = 0; // Truncate to directory
                            iniPath = std::string(pluginPath) + CUSTOM_INI_FILENAME;
                        }
                    }

                    _plugin_logprintf("[%s] Settings file: %s\n", PLUGIN_NAME, iniPath.c_str());
                    _plugin_logprintf("[%s] To edit settings, restart x64dbg with GUI support\n", PLUGIN_NAME);
                }
            }
            break;

        case MENU_HELP:
            DebugLog("[%s] Menu: Help selected\n", PLUGIN_NAME);
            _plugin_logprintf("\n[%s] Help:\n", PLUGIN_NAME);
            _plugin_logprintf("This plugin logs instructions during debugging:\n");
            _plugin_logprintf("1. CALLs - Log function calls\n");
            _plugin_logprintf("2. JMPs - Log jumps\n");
            _plugin_logprintf("3. RETs - Log returns\n\n");
            _plugin_logprintf("Usage:\n");
            _plugin_logprintf("- 'Start Logging' - Begin recording instructions\n");
            _plugin_logprintf("- 'Stop Logging' - Stop recording\n");
            _plugin_logprintf("- 'Clear Log' - Clear the instruction log\n");
            _plugin_logprintf("- 'Save Log' - Save log to a file\n");
            _plugin_logprintf("- 'Settings' - Configure logging options\n\n");

            // Get INI file path
            char pluginPath[MAX_PATH] = "";
            std::string iniPath = "Unknown Location";

            if (GetModuleFileNameA(GetModuleHandleA("InstrLogger_x64.dp64"), pluginPath, MAX_PATH) ||
                GetModuleFileNameA(GetModuleHandleA("InstrLogger_x32.dp32"), pluginPath, MAX_PATH))
            {
                char *lastSlash = strrchr(pluginPath, '\\');
                if (lastSlash)
                {
                    *(lastSlash + 1) = 0; // Truncate to directory
                    iniPath = std::string(pluginPath) + "InstrLoggerSettings.ini";
                }
            }

            _plugin_logprintf("Settings are automatically saved to:\n");
            _plugin_logprintf("  %s\n\n", iniPath.c_str());
            _plugin_logprintf("This allows your preferences to persist between x64dbg sessions.\n");
            break;
        }
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBMENUENTRY: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBMENUENTRY\n", PLUGIN_NAME);
    }
}

// Window event callback
void CBWINEVENT(CBTYPE cbType, PLUG_CB_WINEVENT *info)
{
    try
    {
        // Handle window events if needed
        // Currently not used, but keeping for potential future use
    }
    catch (const std::exception &e)
    {
        DebugLog("[%s] Exception in CBWINEVENT: %s\n", PLUGIN_NAME, e.what());
    }
    catch (...)
    {
        DebugLog("[%s] Unknown exception in CBWINEVENT\n", PLUGIN_NAME);
    }
}

// DLL entry point
BOOL APIENTRY DllMain(HMODULE hModule, DWORD reason, LPVOID reserved)
{
    switch (reason)
    {
    case DLL_PROCESS_ATTACH:
        // Don't log here as the logger isn't initialized yet
        break;
    case DLL_PROCESS_DETACH:
        // Don't log here as the logger may be destroyed already
        break;
    case DLL_THREAD_ATTACH:
        break;
    case DLL_THREAD_DETACH:
        break;
    }
    return TRUE;
}

// Function to create the settings dialog dynamically
HWND CreateSettingsDialog(HWND hWndParent)
{
    // Calculate dialog dimensions
    int dlgWidth = 740;  // Увеличено с 700
    int dlgHeight = 720; // Увеличено с 680
    int groupBoxWidth = dlgWidth - 20;
    int controlWidth = groupBoxWidth - 30;
    int checkboxHeight = 20;
    int editHeight = 24;
    int buttonWidth = 90;
    int buttonHeight = 28;
    int margin = 10;
    int groupPadding = 25; // Space inside group boxes

    // Register dialog class if not yet registered
    static bool classRegistered = false;
    if (!classRegistered)
    {
        WNDCLASSEXA wc = {0};
        wc.cbSize = sizeof(WNDCLASSEXA);
        wc.lpfnWndProc = SettingsDlgProc;
        wc.hInstance = GetModuleHandle(NULL);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.lpszClassName = "INSTRLOGGER_DIALOG";
        wc.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
        // Add double-buffering to prevent flickering during scrolling
        wc.style = CS_HREDRAW | CS_VREDRAW;

        if (!RegisterClassExA(&wc))
        {
            DWORD error = GetLastError();
            if (error != ERROR_CLASS_ALREADY_EXISTS)
            {
                DebugLog("[%s] Failed to register dialog class (error: %d)\n", PLUGIN_NAME, error);
                return NULL;
            }
        }

        classRegistered = true;
        DebugLog("[%s] Dialog class registered\n", PLUGIN_NAME);
    }

    // Create the dialog window with scrolling capability
    HWND hDlg = CreateWindowExA(
        WS_EX_DLGMODALFRAME | WS_EX_TOPMOST,
        "INSTRLOGGER_DIALOG",          // Dialog class
        "Instruction Logger Settings", // Dialog title
        WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_VSCROLL | WS_VISIBLE | WS_OVERLAPPED,
        CW_USEDEFAULT, CW_USEDEFAULT,
        dlgWidth, dlgHeight,
        hWndParent,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    if (!hDlg)
    {
        DebugLog("[%s] Failed to create dialog window (error: %d)\n", PLUGIN_NAME, GetLastError());
        return NULL;
    }

    // Get the dialog's font
    HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);

    // Get a nicer font for group boxes
    HFONT hFontBold = CreateFontA(
        -14, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE,
        DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
        DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Segoe UI");

    // Create Group Box for Instruction Types
    HWND hInstrGroup = CreateWindowExA(
        0,
        "BUTTON",
        "Instruction Types to Log",
        WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        margin, margin,
        groupBoxWidth, 130, // Увеличено с 120
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create checkboxes in Instruction Types group
    HWND hCallsCheck = CreateWindowExA(
        0,
        "BUTTON",
        "CALL instructions",
        WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
        margin + 15, margin + 35, // Увеличено
        controlWidth, checkboxHeight,
        hDlg,
        (HMENU)IDC_CALLS_CHECK,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    HWND hJumpsCheck = CreateWindowExA(
        0,
        "BUTTON",
        "JMP instructions",
        WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
        margin + 15, margin + 35 + (checkboxHeight + 15), // Увеличено расстояние
        controlWidth, checkboxHeight,
        hDlg,
        (HMENU)IDC_JUMPS_CHECK,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    HWND hReturnsCheck = CreateWindowExA(
        0,
        "BUTTON",
        "RET instructions",
        WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
        margin + 15, margin + 35 + (checkboxHeight + 15) * 2, // Увеличено расстояние
        controlWidth, checkboxHeight,
        hDlg,
        (HMENU)IDC_RETURNS_CHECK,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create Group Box for Filtering Options
    HWND hFilterGroup = CreateWindowExA(
        0,
        "BUTTON",
        "Filtering Options",
        WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        margin, margin + 160, // Увеличено с 140
        groupBoxWidth, 75,    // Увеличено с 70
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create checkboxes in Filtering Options group
    HWND hModuleCheck = CreateWindowExA(
        0,
        "BUTTON",
        "Log only instructions from current module",
        WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
        margin + 15, margin + 190, // Увеличено с 165
        controlWidth, checkboxHeight,
        hDlg,
        (HMENU)IDC_MODULE_CHECK,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create Group Box for Display Options
    HWND hDisplayGroup = CreateWindowExA(
        0,
        "BUTTON",
        "Display Options",
        WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        margin, margin + 250, // Увеличено с 220
        groupBoxWidth, 75,    // Увеличено с 70
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create checkbox in Display Options group
    HWND hFollowCheck = CreateWindowExA(
        0,
        "BUTTON",
        "Follow instruction destination in CPU view",
        WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
        margin + 15, margin + 280, // Увеличено с 245
        controlWidth, checkboxHeight,
        hDlg,
        (HMENU)IDC_FOLLOW_CHECK,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create Group Box for File Output
    HWND hFileGroup = CreateWindowExA(
        0,
        "BUTTON",
        "File Output Settings",
        WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        margin, margin + 340, // Увеличено с 300
        groupBoxWidth, 110,   // Увеличено с 100
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create controls in File Output group
    HWND hFileCheck = CreateWindowExA(
        0,
        "BUTTON",
        "Log to file",
        WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
        margin + 15, margin + 370, // Увеличено с 325
        controlWidth / 2, checkboxHeight,
        hDlg,
        (HMENU)IDC_FILE_CHECK,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create static label for file path
    HWND hFileLabel = CreateWindowExA(
        0,
        "STATIC",
        "Output file path:",
        WS_CHILD | WS_VISIBLE,
        margin + 15, margin + 370 + checkboxHeight + 10, // Увеличено
        120, 20,
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create file path edit control
    HWND hFilePath = CreateWindowExA(
        WS_EX_CLIENTEDGE,
        "EDIT",
        "",
        WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL,
        margin + 15 + 120, margin + 370 + checkboxHeight + 8, // Увеличено
        groupBoxWidth - 120 - 115, editHeight,
        hDlg,
        (HMENU)IDC_FILE_PATH,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create browse button with better styling
    HWND hBrowseBtn = CreateWindowExA(
        0,
        "BUTTON",
        "Browse...",
        WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON | BS_ICON,
        margin + groupBoxWidth - 95, margin + 370 + checkboxHeight + 7, // Увеличено
        80, buttonHeight,
        hDlg,
        (HMENU)IDC_BROWSE_BTN,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Add help text under the file path
    HWND hFileHelpText = CreateWindowExA(
        0,
        "STATIC",
        "Logs will be saved in CSV format for easy analysis",
        WS_CHILD | WS_VISIBLE,
        margin + 15, margin + 370 + checkboxHeight + 38, // Увеличено
        groupBoxWidth - 30, 20,
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Calculate the position for the bottom buttons area - position them lower in the larger dialog
    int bottomAreaY = dlgHeight - margin - buttonHeight - 25; // Increased space from bottom

    // Create a separator line above the buttons
    HWND hSeparator = CreateWindowExA(
        0,
        "STATIC",
        "",
        WS_CHILD | WS_VISIBLE | SS_ETCHEDHORZ,
        margin, bottomAreaY - 10,
        dlgWidth - margin * 2, 2,
        hDlg,
        NULL,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Create OK and Cancel buttons with better styling - make them fixed at the bottom
    HWND hOkBtn = CreateWindowExA(
        0,
        "BUTTON",
        "Save Settings",
        WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON | BS_DEFPUSHBUTTON,
        dlgWidth - margin - buttonWidth * 2 - 30, bottomAreaY,
        buttonWidth + 20, buttonHeight,
        hDlg,
        (HMENU)IDC_OK_BTN,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    HWND hCancelBtn = CreateWindowExA(
        0,
        "BUTTON",
        "Cancel",
        WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
        dlgWidth - margin - buttonWidth, bottomAreaY,
        buttonWidth, buttonHeight,
        hDlg,
        (HMENU)IDC_CANCEL_BTN,
        (HINSTANCE)GetModuleHandle(NULL),
        NULL);

    // Add tooltips for all controls
    HWND hTooltip = CreateWindowExA(
        0, TOOLTIPS_CLASSA, NULL,
        WS_POPUP | TTS_ALWAYSTIP,
        CW_USEDEFAULT, CW_USEDEFAULT,
        CW_USEDEFAULT, CW_USEDEFAULT,
        hDlg, NULL, (HINSTANCE)GetModuleHandle(NULL), NULL);

    if (hTooltip)
    {
        TOOLINFOA ti = {0};
        ti.cbSize = sizeof(TOOLINFOA);
        ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
        ti.hwnd = hDlg;

        // Browse button tooltip
        ti.uId = (UINT_PTR)hBrowseBtn;
        ti.lpszText = "Select a file to save instruction logs";
        SendMessageA(hTooltip, TTM_ADDTOOLA, 0, (LPARAM)&ti);

        // Checkbox tooltips
        ti.uId = (UINT_PTR)hCallsCheck;
        ti.lpszText = "Log all CALL instructions during execution";
        SendMessageA(hTooltip, TTM_ADDTOOLA, 0, (LPARAM)&ti);

        ti.uId = (UINT_PTR)hJumpsCheck;
        ti.lpszText = "Log all JMP instructions during execution";
        SendMessageA(hTooltip, TTM_ADDTOOLA, 0, (LPARAM)&ti);

        ti.uId = (UINT_PTR)hReturnsCheck;
        ti.lpszText = "Log all RET instructions during execution";
        SendMessageA(hTooltip, TTM_ADDTOOLA, 0, (LPARAM)&ti);

        // Button tooltips
        ti.uId = (UINT_PTR)hOkBtn;
        ti.lpszText = "Save settings and close dialog";
        SendMessageA(hTooltip, TTM_ADDTOOLA, 0, (LPARAM)&ti);

        ti.uId = (UINT_PTR)hCancelBtn;
        ti.lpszText = "Close dialog without saving";
        SendMessageA(hTooltip, TTM_ADDTOOLA, 0, (LPARAM)&ti);
    }

    // Set font for all controls
    SendMessage(hInstrGroup, WM_SETFONT, (WPARAM)hFontBold, MAKELPARAM(TRUE, 0));
    SendMessage(hFilterGroup, WM_SETFONT, (WPARAM)hFontBold, MAKELPARAM(TRUE, 0));
    SendMessage(hDisplayGroup, WM_SETFONT, (WPARAM)hFontBold, MAKELPARAM(TRUE, 0));
    SendMessage(hFileGroup, WM_SETFONT, (WPARAM)hFontBold, MAKELPARAM(TRUE, 0));

    SendMessage(hCallsCheck, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hJumpsCheck, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hReturnsCheck, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hModuleCheck, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hFollowCheck, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hFileCheck, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hFileLabel, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hFilePath, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hBrowseBtn, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hOkBtn, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));
    SendMessage(hCancelBtn, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));

    // Initialize checkboxes directly before showing the dialog - fixes checkboxes not showing as checked
    if (g_instrLogger)
    {
        const LogOptions &options = g_instrLogger->GetOptions();
        DebugLog("[%s] Setting initial checkbox states directly during dialog creation\n", PLUGIN_NAME);

        // Set checkboxes directly using BM_SETCHECK
        SendMessage(hCallsCheck, BM_SETCHECK, options.logCalls ? BST_CHECKED : BST_UNCHECKED, 0);
        SendMessage(hJumpsCheck, BM_SETCHECK, options.logJumps ? BST_CHECKED : BST_UNCHECKED, 0);
        SendMessage(hReturnsCheck, BM_SETCHECK, options.logReturns ? BST_CHECKED : BST_UNCHECKED, 0);
        SendMessage(hModuleCheck, BM_SETCHECK, options.logOnlyCurrentModule ? BST_CHECKED : BST_UNCHECKED, 0);
        SendMessage(hFollowCheck, BM_SETCHECK, options.followInCpu ? BST_CHECKED : BST_UNCHECKED, 0);
        SendMessage(hFileCheck, BM_SETCHECK, options.logToFile ? BST_CHECKED : BST_UNCHECKED, 0);

        // Set file path
        SetWindowTextA(hFilePath, options.outputFile.c_str());

        // Enable/disable file path controls
        EnableWindow(hFilePath, options.logToFile);
        EnableWindow(hBrowseBtn, options.logToFile);

        DebugLog("[%s] Initial checkboxes set: CALL=%d, JMP=%d, RET=%d, ModuleOnly=%d, Follow=%d, LogToFile=%d\n",
                 PLUGIN_NAME,
                 options.logCalls ? 1 : 0,
                 options.logJumps ? 1 : 0,
                 options.logReturns ? 1 : 0,
                 options.logOnlyCurrentModule ? 1 : 0,
                 options.followInCpu ? 1 : 0,
                 options.logToFile ? 1 : 0);
    }

    // Values will be set in the WM_INITDIALOG handler

    // Setup scrollbar for vertical scrolling
    // Calculate total content height (position of the bottom-most control + its height + margin)
    int contentHeight = margin + 370 + checkboxHeight + 38 + 20 + margin; // Calculate based on the last control position

    // Add extra space to ensure all controls are visible when scrolled to the bottom
    contentHeight += 200; // Increased to ensure buttons are visible in the larger dialog

    // Set up scroll bar range and page size
    SCROLLINFO si = {0};
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_ALL;
    si.nMin = 0;
    si.nMax = contentHeight;
    si.nPage = dlgHeight;
    si.nPos = 0;
    SetScrollInfo(hDlg, SB_VERT, &si, TRUE);

    // Update static variables in the dialog procedure
    SendMessage(hDlg, WM_USER + 100, 0, contentHeight); // Update max scroll position

    // Make sure the buttons are always visible by bringing them to the front
    SetWindowPos(hOkBtn, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
    SetWindowPos(hCancelBtn, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

    // Force a redraw to ensure everything is visible
    InvalidateRect(hDlg, NULL, TRUE);
    UpdateWindow(hDlg);

    return hDlg;
}

// Function to display the settings dialog
void ShowSettingsDialog(HWND hWndParent)
{
    DebugLog("[%s] Creating settings dialog (parent window: %p)\n", PLUGIN_NAME, (void *)hWndParent);

    // Create the dialog
    HWND hDlg = CreateSettingsDialog(hWndParent);
    if (!hDlg)
    {
        DebugLog("[%s] Failed to create settings dialog\n", PLUGIN_NAME);
        return;
    }

    // Store handle in a thread-safe way (using a pointer instead of an integer)
    g_hSettingsDialog = (int)(LONG_PTR)hDlg;
    DebugLog("[%s] Settings dialog created (handle: %p)\n", PLUGIN_NAME, (void *)hDlg);

    // Center the dialog on the parent window
    RECT rcParent, rcDlg;
    GetWindowRect(hWndParent, &rcParent);
    GetWindowRect(hDlg, &rcDlg);
    int x = rcParent.left + (rcParent.right - rcParent.left - (rcDlg.right - rcDlg.left)) / 2;
    int y = rcParent.top + (rcParent.bottom - rcParent.top - (rcDlg.bottom - rcDlg.top)) / 2;
    SetWindowPos(hDlg, NULL, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
    DebugLog("[%s] Settings dialog positioned at (%d, %d)\n", PLUGIN_NAME, x, y);

    // Show the dialog as modal
    ShowWindow(hDlg, SW_SHOW);
    UpdateWindow(hDlg);

    // Disable parent window to create a modal-like behavior
    EnableWindow(hWndParent, FALSE);

    DebugLog("[%s] Settings dialog displayed\n", PLUGIN_NAME);

    // More robust message loop for the dialog
    MSG msg;
    DebugLog("[%s] Starting settings dialog message loop\n", PLUGIN_NAME);
    BOOL bRet;

    // Новый цикл обработки сообщений, который корректно закончится при закрытии диалога
    while ((bRet = GetMessage(&msg, NULL, 0, 0)) != 0)
    {
        // Проверка, существует ли ещё диалог
        if (!IsWindow(hDlg))
        {
            DebugLog("[%s] Dialog window no longer exists, exiting message loop\n", PLUGIN_NAME);
            break;
        }

        // Стандартная обработка диалоговых сообщений
        if (!IsDialogMessage(hDlg, &msg))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    // Проверка на ошибку в GetMessage
    if (bRet == -1)
    {
        DebugLog("[%s] GetMessage returned -1, error code: %d\n", PLUGIN_NAME, GetLastError());
    }

    DebugLog("[%s] Settings dialog message loop exited\n", PLUGIN_NAME);

    // Re-enable parent window
    EnableWindow(hWndParent, TRUE);
    SetForegroundWindow(hWndParent);

    g_hSettingsDialog = 0;
    DebugLog("[%s] Settings dialog closed\n", PLUGIN_NAME);
}

// Проверяет, является ли инструкция JMP
bool IsJumpInstruction(const unsigned char* data, size_t size) {
    if (!data || size < 1)
        return false;

    // JMP rel32 (E9 cd)
    if (size >= 5 && data[0] == 0xE9)
        return true;

    // JMP rel8 (EB cb)
    if (size >= 2 && data[0] == 0xEB)
        return true;

    // JMP r/m32 (FF /4)
    if (size >= 2 && data[0] == 0xFF && (data[1] & 0x38) == 0x20)
        return true;

    return false;
}

// Проверяет, является ли инструкция CALL
bool IsCallInstruction(const unsigned char* data, size_t size) {
    if (!data || size < 1)
        return false;

    // CALL rel32 (E8 cd)
    if (size >= 5 && data[0] == 0xE8)
        return true;

    // CALL r/m32 (FF /2)
    if (size >= 2 && data[0] == 0xFF && (data[1] & 0x38) == 0x10)
        return true;

    // CALL m32 (FF /3)
    if (size >= 2 && data[0] == 0xFF && (data[1] & 0x38) == 0x18)
        return true;

    return false;
}

// Проверяет, является ли инструкция RET
bool IsRetInstruction(const unsigned char* data, size_t size) {
    if (!data || size < 1)
        return false;

    // RET (C3)
    if (size >= 1 && data[0] == 0xC3)
        return true;

    // RET imm16 (C2 iw)
    if (size >= 3 && data[0] == 0xC2)
        return true;

    // RET far (CB)
    if (size >= 1 && data[0] == 0xCB)
        return true;

    // RET far imm16 (CA iw)
    if (size >= 3 && data[0] == 0xCA)
        return true;

    return false;
}
